/****************************************************************
 * Paint Shop Pro File Format Library
 * Library function declarations
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: pspproto.h,v 1.1.1.1 1999/08/15 19:54:41 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

#ifndef __PSPPROTO_H
#define __PSPPROTO_H 1

/****************************************************************
 * PSPOpen
 *
 * Open a PSP file, like the stdio fopen but returns a pointer to
 * the PSP struct instead.
 * NOTE: Only modes "r" and "w" make sense, so expect an
 *       error if another mode is specified. (a following 'b'
 *       should become valid too)
 */

PSP * PSPOpen (const char *,
	       const char *);

/****************************************************************
 * PSPClose
 *
 * Like stdio's fclose but takes a PSP struct pointer as
 * argument
 */

int PSPClose (PSP * psp);

/****************************************************************
 * PSPGetField
 *
 * Read information from the opened PSP file
 * The field is one of the PSPF_* values defined above
 * value is a pointer to the mentioned datatype.
 * space for strings and other values of unknown size is
 * allocated using malloc and should be free()'d when not needed
 * anymore to avoid memory leaks.
 */
int PSPGetField (PSP * psp, pspf_t field, void * value);


/****************************************************************
 * PSPSetLayer
 *
 * Select the next layer for READ operations only.
 * There are additional layer number defines.  See above for a
 * list
 */
int PSPSetLayer (PSP * psp, psplay_t layertype);

/****************************************************************
 * PSPNextLayer
 *
 * Go to the next layer for read/write operations.
 * The effects are identical to PSPSetLayer (psp, PSP_LAYER_NEXT)
 * when loading an image.
 */
int PSPNextLayer (PSP * psp);

/****************************************************************
 * PSPSetChannel
 *
 * Select the channel for the following channel operations
 */
int PSPSetChannel (PSP * psp, pspcht_t channeltype);

/****************************************************************
 * PSPNextChannel
 *
 * Select the following channel for the following channel
 * operations
 */
int PSPNextChannel (PSP * psp);

/****************************************************************
 * PSPReadScanline
 *
 * Read a scanline from the file and place the uncompressed
 * contents in the data array
 */
int PSPReadScanline (PSP * psp, pspbyte_t * data, pspdword_t row);

/****************************************************************
 * PSPGetColorMap
 *
 * Read a colormap entry
 */
int PSPGetColor (PSP * psp, pspdword_t num,
		 pspbyte_t * red, pspbyte_t * green, pspbyte_t * blue);

/****************************************************************
 * PSPMalloc
 *
 * PSP wrapper to the standard C library function malloc ()
 */
void * PSPMalloc (size_t size);

/****************************************************************
 * PSPCalloc
 *
 * PSP wrapper to the standard C library function calloc ()
 */
void * PSPCalloc (size_t nmemb, size_t size);

/****************************************************************
 * PSPFree
 *
 * PSP wrapper to the standard C library function free ()
 */
void PSPFree (void * ptr);

/****************************************************************
 * PSPStrError
 *
 * Return descriptive string for a PSPErrno.  It returns a string
 * descriping the error which occured in libpsp.
 */
char * PSPStrError (int PSPErrnum);

/****************************************************************
 * PSPDebug
 *
 * inizialize debugging. set debuglevel and tell whether the
 * output should go to STDERR (if 0) or to the file
 * /tmp/psp-<pid>.dbg (otherwise)
 */

int PSPDebug (const int debuglevel,
	      const char * tmpl);


#endif /* __PSPPROTO_H */
