/****************************************************************
 * Paint Shop Pro File Format Library
 * Definition of types and attributes
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: psptypes.h,v 1.1.1.1 1999/08/15 19:54:41 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

#ifndef __PSPTYPES_H
#define __PSPTYPES_H 1

/****************************************************************
 * Type defines for the psp library
 */

typedef unsigned long		pspf_t;

typedef int			pspbool_t;
typedef int			pspenum_t;
typedef unsigned char		pspbyte_t;
typedef unsigned short		pspword_t;
typedef unsigned long		pspdword_t;
typedef long			pspcht_t;
typedef long			psplay_t;

struct psprect
{
    long xmin;
    long ymin;
    long xmax;
    long ymax;
};
typedef struct psprect		psprect_t;

struct __pspblndrng
{
    struct
    {
	pspbyte_t low;
	pspbyte_t high;
    } in;
    struct
    {
	pspbyte_t low;
	pspbyte_t high;
    } out;
};
typedef struct pspblndrng
{
    struct __pspblndrng src;
    struct __pspblndrng dst;
}				pspblndrng;

/****************************************************************
 * Data Fields in the General Image Attributes Block
 */

#define PSPB_GENERAL		0x10000

#define PSPF_IMAGEWIDTH		(   1 | PSPB_GENERAL) /* long */
#define PSPF_IMAGEHEIGHT	(   2 | PSPB_GENERAL) /* long */
    /* Get Resolution may fail and set PSPErrno to PSPENIMPL if not
     * supported.  A default value of 72.0 per unit is returned in
     * that case
     */
#define PSPF_IMAGERESVALUE	(   3 | PSPB_GENERAL) /* double */
#define PSPF_IMAGERESUNIT	(   4 | PSPB_GENERAL) /* pspenum_t, see below*/
#define PSPF_IMAGECOMPRTYPE	(   5 | PSPB_GENERAL) /* pspenum_t, see below*/
#define PSPF_IMAGEBITDEPTH	(   6 | PSPB_GENERAL) /* unsigned short */
/* #define PSPF_IMAGEPLANECOUNT	(   7 | PSPB_GENERAL) not exported, always 1*/
#define PSPF_IMAGECOLORCOUNT	(   8 | PSPB_GENERAL) /* pspdword_t */
#define PSPF_IMAGEISGREYSCALE	(   9 | PSPB_GENERAL) /* pspbool_t */
/* #define PSPF_IMAGETOTALSIZE	(  10 | PSPB_GENERAL) */
#define PSPF_IMAGEACTIVELAYER	(  11 | PSPB_GENERAL) /* long */
#define PSPF_IMAGELAYERCOUNT	(  12 | PSPB_GENERAL) /* long */

/* Valid values of the PSPF_IMAGERESUNIT */
#define PSP_IMAGERESUNIT_UNDEFINED	0
#define PSP_IMAGERESUNIT_INCH		1
#define PSP_IMAGERESUNIT_CM		2

/* Valid values for PSPF_IMAGECOMPRTYPE */
#define PSP_IMAGE_COMPR_NONE		0
#define PSP_IMAGE_COMPR_RLE		1
    /* Note: LZ77 compression/decompression not supported yet */
#define PSP_IMAGE_COMPR_LZ77		2

/****************************************************************
 * Data Fields in the Creator Block
 */

#define PSPB_CREATOR		0x40000

#define PSPF_CREATORTITLE	(   1 | PSPB_CREATOR) /* char * */
#define PSPF_CREATORCDATE	(   2 | PSPB_CREATOR) /* time_t */
/* #define PSPF_CREATORMDATE	(   3 | PSPB_CREATOR) time_t */
#define PSPF_CREATORARTIST	(   4 | PSPB_CREATOR) /* char * */
#define PSPF_CREATORCOPYRIGHT	(   5 | PSPB_CREATOR) /* char * */
#define PSPF_CREATORDESC	(   6 | PSPB_CREATOR) /* char * */
#define PSPF_CREATORAPPID	(   7 | PSPB_CREATOR) /* pspdword_t */
#define PSPF_CREATORAPPVER	(   8 | PSPB_CREATOR) /* pspword_t[2] */
#define PSPF_CREATORAPP		(   9 | PSPB_CREATOR) /* char * */

/****************************************************************
 * Data Fields in the Layer Sub-Blocks
 */

#define PSPB_LAYER		0x70000

#define PSPF_LAYERNAME		(   1 | PSPB_LAYER) /* char * */
#define PSPF_LAYERTYPE		(   2 | PSPB_LAYER) /* psplay_t, see below */
#define PSPF_LAYEREXTENT	(   3 | PSPB_LAYER) /* psprect_t */
#define PSPF_LAYERDATAEXTENT	(   4 | PSPB_LAYER) /* psprect_t */
#define PSPF_LAYEROPACITY	(   5 | PSPB_LAYER) /* pspbyte_t */
#define PSPF_LAYERMODE		(   6 | PSPB_LAYER) /* pspenum_t, see below */
#define PSPF_LAYERVISIBLE	(   7 | PSPB_LAYER) /* pspbool_t */
#define PSPF_LAYERTRANSPROT	(   8 | PSPB_LAYER) /* pspbool_t */
#define PSPF_LAYERLINKGROUP	(   9 | PSPB_LAYER) /* pspword_t */
#define PSPF_LAYERMASKEXTENT	(  10 | PSPB_LAYER) /* psprect_t */
#define PSPF_LAYERMASKDATAEXTENT (  11 | PSPB_LAYER) /* psprect_t */
#define PSPF_LAYERMASKLINKED	(  12 | PSPB_LAYER) /* pspbool_t */
#define PSPF_LAYERMASKDISABLED	(  13 | PSPB_LAYER) /* pspbool_t */
#define PSPF_LAYERMASKINVERT	(  14 | PSPB_LAYER) /* pspbool_t */
/* #define PSPF_LAYERBLENDRCOUNT	(  15 | PSPB_LAYER)  pspword_t */
/* #define PSPF_LAYERBLENDRANGE1	(  16 | PSPB_LAYER)  pspblndrng_t */
/* #define PSPF_LAYERBLENDRANGE2	(  17 | PSPB_LAYER)  pspblndrng_t */
/* #define PSPF_LAYERBLENDRANGE3	(  18 | PSPB_LAYER)  pspblndrng_t */
/* #define PSPF_LAYERBLENDRANGE4	(  19 | PSPB_LAYER)  pspblndrng_t */
/* #define PSPF_LAYERBLENDRANGE5	(  20 | PSPB_LAYER)  pspblndrng_t */
#define PSPF_LAYERBITMAPCOUNT	(  21 | PSPB_LAYER) /* pspword_t */
#define PSPF_LAYERCHANNELCOUNT	(  22 | PSPB_LAYER) /* pspword_t */
#define PSPF_LAYERNUMBER	(  23 | PSPB_LAYER) /* psplay_t */
#define PSPF_LAYERSCANLINEBYTES	(  24 | PSPB_LAYER) /* pspdword_t */
#define PSPF_LAYERSCANLINEALIGN	(  25 | PSPB_LAYER) /* pspdword_t */
#define PSPF_LAYERMASKSCANLINEBYTES	(  26 | PSPB_LAYER) /* pspdword_t */
#define PSPF_LAYERMASKSCANLINEALIGN	(  27 | PSPB_LAYER) /* pspdword_t */

/* Valid values for PSPF_LAYERTYPE */
#define PSP_LAYER_TYPE_NORMAL		0
#define PSP_LAYER_TYPE_FLOATSEL		1

/****************************************************************
 * Selection of layers used as special layernum argument to
 * PSPSetLayer
 */
#define PSP_LAYERNORMALOFFSET	-256
#define PSP_LAYERFLOATOFFSET	-512
#define PSP_LAYERALPHAOFFSET	-768

#define PSP_LAYERFIRST		-1L
#define PSP_LAYERLAST		-2L
#define PSP_LAYERNEXT		-3L
#define PSP_LAYERPREV		-4L

#define PSP_LAYERFIRSTNORMAL	-1L + PSP_LAYERNORMALOFFSET
#define PSP_LAYERLASTNORMAL	-2L + PSP_LAYERNORMALOFFSET
#define PSP_LAYERNEXTNORMAL	-3L + PSP_LAYERNORMALOFFSET
#define PSP_LAYERPREVNORMAL	-4L + PSP_LAYERNORMALOFFSET

#define PSP_LAYERFIRSTFLOAT	-1L + PSP_LAYERFLOATOFFSET
#define PSP_LAYERLASTFLOAT	-2L + PSP_LAYERFLOATOFFSET
#define PSP_LAYERNEXTFLOAT	-3L + PSP_LAYERFLOATOFFSET
#define PSP_LAYERPREVFLOAT	-4L + PSP_LAYERFLOATOFFSET

#define PSP_LAYERFIRSTALPHA	-1L + PSP_LAYERALPHAOFFSET
#define PSP_LAYERLASTALPHA	-2L + PSP_LAYERALPHAOFFSET
#define PSP_LAYERNEXTALPHA	-3L + PSP_LAYERALPHAOFFSET
#define PSP_LAYERPREVALPHA	-4L + PSP_LAYERALPHAOFFSET

/****************************************************************
 * Layer blending modes supported by PSP
 * (values returned by a PSPF_LAYERMODE request)
 */

#define PSP_LAYERMODE_NORMAL		0
#define PSP_LAYERMODE_DARKEN		1
#define PSP_LAYERMODE_LIGHTEN		2
#define PSP_LAYERMODE_HUE		3
#define PSP_LAYERMODE_SATURATION	4
#define PSP_LAYERMODE_COLOR		5
#define PSP_LAYERMODE_LUMINOSITY	6
#define PSP_LAYERMODE_MULTIPLY		7
#define PSP_LAYERMODE_SCREEN		8
#define PSP_LAYERMODE_DISSOLVE		9
#define PSP_LAYERMODE_OVERLAY		10
#define PSP_LAYERMODE_HARDLIGHT		11
#define PSP_LAYERMODE_SOFTLIGHT		12
#define PSP_LAYERMODE_DIFFERENCE	13
#define PSP_LAYERMODE_DODGE		14
#define PSP_LAYERMODE_BURN		15
#define PSP_LAYERMODE_EXCLUSION		16

/****************************************************************
 * Data Fields in the Channel Sub-Blocks
 */

#define PSPB_CHANNEL		0x1000000

#define PSPF_CHANNELTYPE	(   1 | PSPB_CHANNEL) /* pspcht_t */
#define PSPF_CHANNELNUMBER	(   2 | PSPB_CHANNEL) /* pspcht_t */
#define PSPF_CHANNELBITMAPTYPE	(   3 | PSPB_CHANNEL) /* pspword_t */
#define PSPF_CHANNELCHANNELTYPE	(   4 | PSPB_CHANNEL) /* pspword_t */

/****************************************************************
 * Selection of channels used for PSPSetChannel, PSPGetChannel
 */

#define PSP_CHANNELNONE			0 /* Only as return PSPGetChannel
					     allowed */
#define PSP_CHANNELSELECTIONOFFSET	-256
#define PSP_CHANNELALPHAOFFSET		-512

#define PSP_CHANNELFIRST		-1
#define PSP_CHANNELLAST			-2
#define PSP_CHANNELNEXT			-3
#define PSP_CHANNELPREV			-4
#define PSP_CHANNELCOMPOSITE		-5
#define PSP_CHANNELRED			-6
#define PSP_CHANNELGREEN		-7
#define PSP_CHANNELBLUE			-8
#define PSP_CHANNELTRANSMASK		-9
#define PSP_CHANNELUSERMASK		-10

#define PSP_CHANNELSELECTION		-11 + PSP_CHANNELSELECTIONOFFSET
/* These are not really useful at the moment as only one channel block
 * occurs within each alpha channel as there is only 1 channel */
#define PSP_CHANNELSELECTIONFIRST	-12 + PSP_CHANNELSELECTIONOFFSET
#define PSP_CHANNELSELECTIONLAST	-13 + PSP_CHANNELSELECTIONOFFSET
#define PSP_CHANNELSELECTIONNEXT	-14 + PSP_CHANNELSELECTIONOFFSET
#define PSP_CHANNELSELECTIONPREV	-15 + PSP_CHANNELSELECTIONOFFSET

#define PSP_CHANNELALPHA		-16 + PSP_CHANNELALPHAOFFSET
/* These are not really useful at the moment as only one channel block
 * occurs within each alpha channel */
#define PSP_CHANNELALPHAFIRST		-17 + PSP_CHANNELALPHAOFFSET
#define PSP_CHANNELALPHALAST		-18 + PSP_CHANNELALPHAOFFSET
#define PSP_CHANNELALPHANEXT		-19 + PSP_CHANNELALPHAOFFSET
#define PSP_CHANNELALPHAPREV		-20 + PSP_CHANNELALPHAOFFSET

/****************************************************************
 * Data Fields in the Colormap Sub-Blocks
 */

#define PSPB_CMAP		0x60000

#define PSPF_CMAPCOLORCOUNT	(   1 | PSPB_CMAP) /* pspdword_t */


/****************************************************************
 * Data Fields in the Selection Blocks
 */

#define PSPB_SELECTION		0x80000

#define PSPF_SELECTIONEXTENT		(   1 | PSPB_SELECTION) /* psprect_t */
#define PSPF_SELECTIONDATAEXTENT	(   2 | PSPB_SELECTION) /* psprect_t */
#define PSPF_SELECTIONBITMAPCOUNT	(   3 | PSPB_SELECTION) /* pspword_t */
#define PSPF_SELECTIONCHANNELCOUNT	(   4 | PSPB_SELECTION) /* pspword_t */
#define PSPF_SELECTIONSCANLINEBYTES	(   5 | PSPB_SELECTION) /* pspdword_t*/
#define PSPF_SELECTIONSCANLINEALIGN	(   6 | PSPB_SELECTION) /* pspdword_t*/

/****************************************************************
 * Data Fields in the Alpha Blocks
 * These are just redefinitions as Alpha Channels are not
 * unlike layers and therefore treated quite similar.
 */

#define PSPF_ALPHANAME			PSPF_LAYERNAME          /* psprect_t */
#define PSPF_ALPHAEXTENT		PSPF_LAYEREXTENT        /* psprect_t */
#define PSPF_ALPHADATAEXTENT		PSPF_LAYERDATAEXTENT    /* psprect_t */
#define PSPF_ALPHABITMAPCOUNT		PSPF_LAYERBITMAPCOUNT   /* pspword_t */
#define PSPF_ALPHACHANNELCOUNT		PSPF_LAYERCHANNELCOUNT  /* pspword_t */
#define PSPF_ALPHASCANLINEBYTES		PSPF_LAYERSCANLINEBYTES /* pspdword_t*/
#define PSPF_ALPHASCANLINEALIGN		PSPF_LAYERSCANLINEALIGN /* pspdword_t*/

#endif /* __PSPTYPES_H */
