/****************************************************************
 * Paint Shop Pro File Format Library
 * Internal function declarations
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: psp-internal-proto.h,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

#ifndef __PSP_LIB_INT_FUNCS_H
#define __PSP_LIB_INT_FUNCS_H 1

#include "psp.h"
#include <pspproto.h>

#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <stdarg.h>


typedef unsigned char psp_int8;
typedef unsigned short psp_int16;
typedef unsigned long psp_int32;
typedef unsigned long long psp_int64;

size_t
PSPread_int8 (FILE * fp,
	      psp_int8 * ptr,
	      size_t count);
size_t
PSPread_int16 (FILE * fp,
	       psp_int16 * ptr,
	       size_t count);
size_t
PSPread_int32 (FILE * fp,
	       psp_int32 * ptr,
	       size_t count);
size_t
PSPread_time_t (FILE * fp,
		time_t * ptr,
		size_t count);
size_t
PSPread_double (FILE * fp,
		double * ptr,
		size_t count);
size_t
PSPwrite_int8 (FILE * fp,
	       const psp_int8 * ptr,
	       size_t count);
size_t
PSPwrite_int16 (FILE * fp,
		const psp_int16 * ptr,
		size_t count);
size_t
PSPwrite_int32 (FILE * fp,
		psp_int32 * ptr,
		size_t count);

psp_int16
psp_itoh16(psp_int16 intel16);
psp_int16
psp_htoi16(psp_int16 host16);
psp_int32
psp_itoh32(psp_int32 intel32);
psp_int32
psp_htoi32(psp_int32 host32);
psp_int32
psp_htoitime_t(time_t itime);
time_t
psp_itohtime_t(psp_int32 itime);
double
psp_itohdouble (psp_int64 dbl);
psp_int64
psp_htoidouble (double dbl);

PSP *
PSPreadinfo (PSP * psp);

struct PSP_GeneralImageAttrInfoChunk *
PSPReadGeneralImageAttrInfo (FILE * fp,
			     struct PSP_GeneralImageAttrInfoChunk * g);
int
PSPGetGeneralField (PSP * psp,
		    pspf_t field,
		    void * value);

struct PSPCreatorInfo *
PSPReadCreatorInfo (FILE * fp,
		    DWORD firstsize,
		    DWORD blocksize,
		    struct PSPCreatorInfo * creator);
int
PSPGetCreatorField (PSP * psp,
		    pspf_t field,
		    void * value);

struct PSPLayerInfo **
PSPReadLayerInfo (PSP * psp, DWORD firstsize, DWORD blocksize);
int
PSPGetLayerField (PSP * psp,
		  pspf_t field,
		  void * value);

struct PSPChannelInfo ***
PSPReadChannelInfo (FILE * fp, DWORD firstsize, DWORD blocksize,
		    struct PSPChannelInfo *** channel, WORD numchannels);
int
PSPGetChannelField (PSP * psp,
		    pspf_t field,
		    void * value);

struct PSPCMapInfo **
PSPReadCMapInfo (FILE * fp,
		 struct PSPCMapInfo ** palette,
		 DWORD blocksize);
int
PSPGetCMapField (PSP * psp,
		 pspf_t field,
		 void * value);

struct PSPSelectionInfo *
PSPReadSelectionInfo (FILE * fp,
		      struct PSPSelectionInfo ** selection,
		      DWORD blocksize);
int
PSPGetSelectionField (PSP * psp,
		      pspf_t field,
		      void * value);

struct PSPAlphaInfo **
PSPReadAlphaInfo (PSP * psp,
		  DWORD firstsizee,
		  DWORD blocksize);
int
PSPGetAlphaField (PSP * psp,
		  pspf_t field,
		  void * value);


#endif /* __PSP_LIB_INTFUNCS_H */
