/****************************************************************
 * Paint Shop Pro File Format Library
 * Internal main headerfile
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: psp.h,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

#ifndef __PSP_H
#define __PSP_H 1

#include "config.h"

#include <stdio.h>

#include <psp-internal.h>

struct PSP 
{
    FILE * fp;
				/* a stdio FILE * as in stdio.h */

    struct PSP_GeneralImageAttrInfoChunk general;
				/* this should in fact be hidden
				 * from the user application
				 */
/* Extended Data not supported yet, only information contained herein
 * at the moment is the transparent color palette index
 */
    struct PSP_ExtendedDataChunk ** extended;
				/* Extended Data Block information pairs
				 */
    
/* Tube not supported 
    struct PSP_TubeDataChunk * tube;
*/

    struct PSPCreatorInfo creator;
				/* Pointer to creator data array
				 */

/* Thumbnails not supported yet, but read information chunk */
    struct PSP_ThumbnailAttrChunk * thumbnail;
				/* Thumbnail Attributes */

    struct PSPCMapInfo * palette;
				/* This contains the Palette for indexed
				 * images
				 */

    struct PSPLayerInfo ** layer;
				/* Pointer to array of layer information
				 */
    
    long layernum;

    WORD alphanum;

    pspbool_t inselection;
    
    long channelnum;
    
    struct PSPSelectionInfo * selection;
				/* Holds information about the selection
				 * which was active when the file was
				 * saved
				 */

    pspbool_t inalpha;

    WORD alphacount;

    struct PSPAlphaInfo ** alpha;
				/* Holds all the information about
				 * channels
				 */

    struct 
    {
	int runtype;
	unsigned char runcount;
	unsigned char lastread;
    } rle;
};

typedef struct PSP PSP;

#include <psp-internal-proto.h>

/****************************************************************
 * PSPfopen
 *
 * Open a PSP file, like the stdio fopen but returns a pointer to
 * the PSP struct instead.
 * NOTE: Only modes "r", "w" and "w+" make sense, so expect an
 *       error if another mode is specified. ("b" should be
 *       made valid too)
 */

PSP * PSPfopen (const char *,
		const char *);

/****************************************************************
 * PSPfclose
 *
 * Like stdio's fclose but takes a PSP struct pointer as
 * argument
 */

int PSPfclose (PSP * psp);

/****************************************************************
 * PSPdebug
 *
 * inizialize debugging. set debuglevel and tell whether the
 * output should go to STDERR (if 0) or to the file
 * /tmp/psp-<pid>.dbg (otherwise)
 */

int PSPdebug (const int debuglevel,
	      const int where);


#endif /* __PSP_H */



