/****************************************************************
 * Paint Shop Pro File Format Library
 * Colormap handling
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: pspcolormap.c,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

static char const rcsid [] = "$Id: pspcolormap.c,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $";

#include "psp.h"
#include <pspproto.h>

#include <debug.h>

#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include <psperrno.h>
int PSPErrno;


struct PSPCMapInfo **
PSPReadCMapInfo (FILE * fp,
		 struct PSPCMapInfo ** palette,
		 DWORD blocksize)
{
    struct PSPCMapInfo * this;
    DWORD numcolors;
    WORD i;

    if ((PSPread_int32 (fp, &numcolors, 1)) != 1)
    {
	debug (16, "Error reading colormap chunk (short file?)!\n");
	return NULL;
    }
	
    if ((*palette = this = PSPLIBMalloc (sizeof (struct PSPCMapInfo) + sizeof
					 (RGBQUAD) * numcolors)) == NULL)
    {
	debug (8, "Unable to allocate memory for Colormap Info\n");
	return NULL;
    }

    this->colors = numcolors;

    debug (32, "Colormap -> count: %lu\n", this->colors);

    for (i = 0; i < this->colors; i++)
    {
	if (PSPread_int8 (fp, &this->cmap[i].red, 1) != 1
	    || PSPread_int8 (fp, &this->cmap[i].green, 1) != 1
	    || PSPread_int8 (fp, &this->cmap[i].blue, 1) != 1
	    || PSPread_int8 (fp, &this->cmap[i].reserved, 1) != 1
	    )
	{
	    debug (16, "Error reading colormap entry %d\n", i);
	    return NULL;
	}
	debug (32, "Colormap -> Entry %d: %02x %02x %02x\n", i,
	       (int) this->cmap[i].red, (int) this->cmap[i].green,
	       (int) this->cmap[i].blue);
    }
    
    return palette;
}

int
PSPGetCMapField (PSP * psp,
		 pspf_t field,
		 void * value)
{
    PSPErrno = PSPENOERR;

    if (psp->palette == NULL)
	return 0;

    switch (field)
    {
      case PSPF_CMAPCOLORCOUNT:
	  *((pspdword_t *) value) = psp->palette->colors;
	  break;
      default:
	  PSPErrno = PSPEINVAL;
	  return 0;
    }
    return 1;
}

int
PSPGetColor (PSP * psp,
	     pspdword_t num,
	     pspbyte_t * red,
	     pspbyte_t * green,
	     pspbyte_t * blue)
{
    RGBQUAD this;
    
    PSPErrno = PSPENOERR;

    if (psp->palette == NULL)
	return 0;

    if (num >= psp->palette->colors)
    {
	PSPErrno = PSPEINVAL;
	return 0;
    }

    this = psp->palette->cmap[num];

    *red = this.red;
    *green = this.green;
    *blue = this.blue;

    return 1;
}
