/****************************************************************
 * Paint Shop Pro File Format Library
 * Selection handling functions
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: pspselection.c,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

static char const rcsid [] = "$Id: pspselection.c,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $";

#include "psp.h"
#include <pspproto.h>

#include <debug.h>

#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <stdarg.h>
#include <errno.h>

#include <psperrno.h>
int PSPErrno;


struct PSPSelectionInfo *
PSPReadSelectionInfo (FILE * fp,
		      struct PSPSelectionInfo ** selection,
		      DWORD blocksize)
{
    struct PSPSelectionInfo * this;

    if ((this = *selection =
	 PSPLIBMalloc (sizeof (struct PSPSelectionInfo))) == NULL)
    {
	debug (8, "Unable to allocate memory for Channel Info\n");
	return NULL;
    }

    if (PSPread_int32 (fp, &this->rect.xmin, 1) != 1
	|| PSPread_int32 (fp, &this->rect.ymin, 1) != 1
	|| PSPread_int32 (fp, &this->rect.xmax, 1) != 1
	|| PSPread_int32 (fp, &this->rect.ymax, 1) != 1
	|| PSPread_int32 (fp, &this->saved_rect.xmin, 1) != 1
	|| PSPread_int32 (fp, &this->saved_rect.ymin, 1) != 1
	|| PSPread_int32 (fp, &this->saved_rect.xmax, 1) != 1
	|| PSPread_int32 (fp, &this->saved_rect.ymax, 1) != 1
	|| PSPread_int16 (fp, &this->bitmaps, 1) != 1
	|| PSPread_int16 (fp, &this->channels, 1) != 1)
    {
	debug (16, "Error reading selection information\n");
	return NULL;
    }

    debug (32, "Selection -> rect: (%ld, %ld), (%ld, %ld)\n",
	   this->rect.xmin, this->rect.ymin,
	   this->rect.xmax, this->rect.ymax);
    debug (32, "Selection -> saved rect: (%ld, %ld), (%ld, %ld)\n",
	   this->saved_rect.xmin, this->saved_rect.ymin,
	   this->saved_rect.xmax, this->saved_rect.ymax);

    debug (32, "Selection -> bitmap count: %hu\n", this->bitmaps);
    debug (32, "Selection -> channel count: %hu\n", this->channels);

	/* Selection is always 8 bit "greyscale" */
    this->scanlinebytes = this->saved_rect.xmax
	- this->saved_rect.xmin;
    this->scanlinealign = this->scanlinebytes;
    this->scanlinealign += 3;
    this->scanlinealign &= 0xfffffffc;
    
    debug (32, "Selection -> scanline bytecount: %lu\n",
	   this->scanlinebytes);
    debug (32, "Selection -> scanline aligned bytecount: %lu\n",
	   this->scanlinealign);

#if 0
    if ( (bmcount != 1) || (chcount != 1) )
    {
	debug (16, "Unsupported selection\n");
	return NULL;
    }
#endif

    if (PSPReadChannelInfo (fp, 0, blocksize, &this->channel, this->channels)
	== NULL)
    {
	debug (16, "Unable to read selection channel(s)\n");
	return NULL;
    }

    return this;
}

int
PSPGetSelectionField (PSP * psp,
		      pspf_t field,
		      void * value)
{
    struct PSPSelectionInfo * selection = psp->selection;

    if (selection == NULL)
    {
	PSPErrno = PSPEINVAL;
	return 0;
    }
    switch (field)
    {
      case PSPF_SELECTIONEXTENT:
	  *((psprect_t *) value) = selection->rect;
	  break;
      case PSPF_SELECTIONDATAEXTENT:
	  *((psprect_t *) value) = selection->saved_rect;
	  break;
      case PSPF_SELECTIONBITMAPCOUNT:
	  *((pspword_t *) value) = selection->bitmaps;
	  break;
      case PSPF_SELECTIONCHANNELCOUNT:
	  *((pspword_t *) value) = selection->channels;
	  break;
      case PSPF_SELECTIONSCANLINEBYTES:
	  *((pspdword_t *) value) = selection->scanlinebytes;
	  break;
      case PSPF_SELECTIONSCANLINEALIGN:
	  *((pspdword_t *) value) = selection->scanlinealign;
	  break;
      default:
	  PSPErrno = PSPEINVAL;
	  return 0;
    }
    return 1;
}
