/****************************************************************
 * Paint Shop Pro File Format Test Program
 *
 * Paint Shop Pro by Jasc Software, Inc. (http://www.jasc.com/)
 *
 * $Id: psp-test.c,v 1.1.1.1 1999/08/15 19:54:42 tl Exp $
 *
 * Copyright (c) 1999
 *               Thomas Linder
 *
 * Based on the psp file-format specs:
 *   http://www.jasc.com/specs/psp.html
 *
 * Inspired by the need to have a format for image interchange
 * between PSP and Gimp.
 ****************************************************************
 * This is part of libpsp and is free software; you can redistribute
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 ****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <psp.h>

#include <psperrno.h>
extern int PSPErrno;

#define MIN(a, b) (((a) < (b)) ? (a) : (b))

char *blendingmodes[] =
{
    "normal",
    "darken",
    "lighten",
    "hue",
    "saturation",
    "color",
    "luminosity",
    "multiply",
    "screen",
    "dissolve",
    "overlay",
    "hard light",
    "soft light",
    "difference",
    "dodge",
    "burn",
    "exclusion"
};

void
testread_channel (PSP * psp)
{
    pspdword_t scanlinebytes;
    psprect_t extent;
    pspdword_t cols, rows;
    pspdword_t col, row;
    pspcht_t cht;
    pspbyte_t * data;

    PSPGetField (psp, PSPF_CHANNELNUMBER, &cht);
    printf ("  . Channel %ld\n", cht);
    PSPGetField (psp, PSPF_CHANNELTYPE, &cht);
    printf ("      . type: ");
    switch (cht)
    {
      case PSP_CHANNELCOMPOSITE:
	  printf ("indexed or grayscale\n");
	  break;
      case PSP_CHANNELRED:
	  printf ("red\n");
	  break;
      case PSP_CHANNELGREEN:
	  printf ("green\n");
	  break;
      case PSP_CHANNELBLUE:
	  printf ("blue\n");
	  break;
      case PSP_CHANNELTRANSMASK:
	  printf ("transparency mask\n");
	  break;
      case PSP_CHANNELUSERMASK:
	  printf ("user mask\n");
	  break;
      case PSP_CHANNELSELECTION:
	  printf ("selection\n");
	  break;
      case PSP_CHANNELALPHA:
	  printf ("alpha mask\n");
	  break;
    }
    switch (cht)
    {
      case PSP_CHANNELCOMPOSITE:
      case PSP_CHANNELRED:
      case PSP_CHANNELGREEN:
      case PSP_CHANNELBLUE:
      case PSP_CHANNELTRANSMASK:
	  if (PSPGetField (psp, PSPF_LAYERSCANLINEBYTES, &scanlinebytes)
	      && PSPGetField (psp, PSPF_LAYERDATAEXTENT, &extent))
	  {
	      cols = extent.xmax - extent.xmin;
	      rows = extent.ymax - extent.ymin;
	      printf ("      . cols (bytes) x rows : %ld (%ld) x %ld\n",
		      cols, scanlinebytes, rows);
	      if ((data = malloc (scanlinebytes)) != NULL)
	      {
		  for (row = 0; row < rows; row++)
		  {
		      if (PSPReadScanline (psp, data, row))
		      {
			  if (row == 0)
			  {
			      printf ("      . begin:");
			      for (col = 0;
				   col < MIN (20, scanlinebytes);
				   col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
			  if (row == rows - 1)
			  {
			      printf ("      . end  :");
			      for (col = (scanlinebytes -
				       (MIN (20, scanlinebytes)));
					col < scanlinebytes;
					col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
		      }
		      else
		      {
			  printf ("What? PSPErrno = %d\n", PSPErrno);
		      }
		  }
		  free (data);
	      }
	  }
	  break;
      case PSP_CHANNELUSERMASK:
	  if (PSPGetField (psp, PSPF_LAYERMASKSCANLINEBYTES, &scanlinebytes)
	      && PSPGetField (psp, PSPF_LAYERMASKDATAEXTENT, &extent))
	  {
	      cols = extent.xmax - extent.xmin;
	      rows = extent.ymax - extent.ymin;
	      printf ("      . cols (bytes) x rows : %ld (%ld) x %ld\n",
		      cols, scanlinebytes, rows);
	  }
	  break;
      case PSP_CHANNELSELECTION:
	  if (PSPGetField (psp, PSPF_SELECTIONSCANLINEBYTES, &scanlinebytes)
	      && PSPGetField (psp, PSPF_SELECTIONDATAEXTENT, &extent))
	  {
	      cols = extent.xmax - extent.xmin;
	      rows = extent.ymax - extent.ymin;
	      printf ("      . cols (bytes) x rows : %ld (%ld) x %ld\n",
		      cols, scanlinebytes, rows);
	      if ((data = malloc (scanlinebytes)) != NULL)
	      {
		  for (row = 0; row < rows; row++)
		  {
		      if (PSPReadScanline (psp, data, row))
		      {
			  if (row == 0)
			  {
			      printf ("      . begin:");
			      for (col = 0;
				   col < MIN (20, scanlinebytes);
				   col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
			  if (row == rows - 1)
			  {
			      printf ("      . end  :");
			      for (col = (scanlinebytes -
				       (MIN (20, scanlinebytes)));
					col < scanlinebytes;
					col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
		      }
		  }
		  free (data);
	      }
	  }
	  break;	  
      case PSP_CHANNELALPHA:
	  if (PSPGetField (psp, PSPF_ALPHASCANLINEBYTES, &scanlinebytes)
	      && PSPGetField (psp, PSPF_ALPHADATAEXTENT, &extent))
	  {
	      cols = extent.xmax - extent.xmin;
	      rows = extent.ymax - extent.ymin;
	      printf ("      . cols (bytes) x rows : %ld (%ld) x %ld\n",
		      cols, scanlinebytes, rows);
	      if ((data = malloc (scanlinebytes)) != NULL)
	      {
		  for (row = 0; row < rows; row++)
		  {
		      if (PSPReadScanline (psp, data, row))
		      {
			  if (row == 0)
			  {
			      printf ("      . begin:");
			      for (col = 0;
				   col < MIN (20, scanlinebytes);
				   col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
			  if (row == rows - 1)
			  {
			      printf ("      . end  :");
			      for (col = (scanlinebytes -
				       (MIN (20, scanlinebytes)));
					col < scanlinebytes;
					col++) 
				  printf (" %02x", data[col]);
			      printf ("\n");
			  }
		      }
		  }
		  free (data);
	      }
	  }
	  break;	  
    }
    return;
}

void
testread (int count, char * mode, char ** names)
{
    int i;
    PSP * psp = NULL;
    long t_32;
    double t_dbl;
    int t_int;
    pspword_t t_u16;
    pspdword_t t_u32;
    pspbyte_t t_u8;
    char * string;
    psprect_t extent;

    for (i = 0; i < count; i++)
    {
	printf ("==== Next file: %s\n", names[i]);
	
	if ((psp = PSPOpen (names[i], mode)) == NULL)
	{
	    printf ("Error opening file %s for reading!\n", names[i]);
	}
	else
	{
	    printf ("Fine! Opened file %s for reading\n\n", names[i]);

	    if (PSPGetField (psp, PSPF_IMAGEWIDTH, &t_32))
		printf ("Image width: %ld\n", t_32);
	    if (PSPGetField (psp, PSPF_IMAGEHEIGHT, &t_32))
		printf ("Image height: %ld\n", t_32);
	    if (PSPGetField (psp, PSPF_IMAGERESVALUE, &t_dbl))
		printf ("Image resolution: %f pixels per unit\n", t_dbl);
	    else if (PSPErrno == PSPENIMPL)
		printf ("Image resolution not supported: default %f "
			"pixels per unit\n", t_dbl);
	    if (PSPGetField (psp, PSPF_IMAGERESUNIT, &t_int))
		printf ("Image resolution units: %d\n", t_int);
	    if (PSPGetField (psp, PSPF_IMAGEBITDEPTH, &t_u16))
		printf ("Image bitdepth: %hu\n", t_u16);
	    if (PSPGetField (psp, PSPF_IMAGECOLORCOUNT, &t_u32))
		printf ("Image colorcount: %lu\n", t_u32);
	    if (PSPGetField (psp, PSPF_IMAGEISGREYSCALE, &t_int))
		printf ("Image is: %s\n", (t_int ? "greyscale" : "color"));
	    if (PSPGetField (psp, PSPF_IMAGELAYERCOUNT, &t_32))
		printf ("Image layer count: %ld\n", t_32);
	    if (PSPGetField (psp, PSPF_IMAGEACTIVELAYER, &t_32))
		printf ("Image active layer: %ld\n\n", t_32);

	    if (PSPGetField (psp, PSPF_CREATORTITLE, &string))
	    {
		printf ("Creator title: %s\n", string);
		PSPFree (string);
	    }
	    if (PSPGetField (psp, PSPF_CREATORARTIST, &string))
	    {
		printf ("Creator artist: %s\n", string);
		PSPFree (string);
	    }
	    if (PSPGetField (psp, PSPF_CREATORCOPYRIGHT, &string))
	    {
		printf ("Creator copyright: %s\n", string);
		PSPFree (string);
	    }
	    if (PSPGetField (psp, PSPF_CREATORDESC, &string))
	    {
		printf ("Creator description: %s\n", string);
		PSPFree (string);
	    }
	    if (PSPGetField (psp, PSPF_CREATORAPPID, &t_u32))
	    {
		printf ("Creator application id: %lu\n", t_u32);
	    }
#if APPVERENABLED
	    if (PSPGetField (psp, PSPF_CREATORAPPVER, &version))
	    {
		printf ("Creator application version: %hu.%hu (%04x.%04x)\n",
			version[0], version[1],
			version[0], version[1]
			);
	    }
#endif /* APPVERENABLED */
	    if (PSPGetField (psp, PSPF_CREATORAPP, &string))
	    {
		printf ("Creator application: %s\n\n", string);
		PSPFree (string);
	    }
	    
	    if (PSPGetField (psp, PSPF_CMAPCOLORCOUNT, &t_u32))
		printf ("Colormap number of entries: %lu\n\n", t_u32);

	    for (i = 0; i < t_u32; i++)
	    {
		pspbyte_t r, g, b;

		if (PSPGetColor (psp, (pspdword_t) i, &r, &g, &b))
		    printf ("  . color %d: %02x %02x %02x\n", i, r, g, b);
	    }
		
	    if (PSPSetLayer (psp, PSP_LAYERFIRST))
	    {
		do {
		    PSPGetField (psp, PSPF_LAYERNUMBER, &t_32);
		    printf ("Layer %ld\n", t_32);
		    if (PSPGetField (psp, PSPF_LAYERNAME, &string))
		    {
			printf ("  . name: %s\n", string);
			PSPFree (string);
		    }
		    if (PSPGetField (psp, PSPF_LAYERTYPE, &t_int))
			printf ("  . type: %d\n", t_int);
		    if (PSPGetField (psp, PSPF_LAYEREXTENT, &extent))
		    {
			printf ("  . extent: (%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPGetField (psp, PSPF_LAYERDATAEXTENT, &extent))
		    {
			printf ("  . saved extent: (%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPGetField (psp, PSPF_LAYEROPACITY, &t_u8))
			printf ("  . opacity: %d\n", (int) t_u8);
		    if (PSPGetField (psp, PSPF_LAYERMODE, &t_int))
		    {
			printf ("  . blending mode: %s\n",
				blendingmodes [t_int]);
		    }
		    if (PSPGetField (psp, PSPF_LAYERVISIBLE, &t_int))
		    {
			printf ("  . visible: %s\n",
				(t_int ? "Yes" : "No"));
		    }
		    if (PSPGetField (psp, PSPF_LAYERTRANSPROT, &t_int))
		    {
			printf ("  . transparency protect: %s\n",
				(t_int ? "Yes" : "No"));
		    }
		    if (PSPGetField (psp, PSPF_LAYERLINKGROUP, &t_u16))
			printf ("  . link group: %hu\n", t_u16);
		    if (PSPGetField (psp, PSPF_LAYERMASKEXTENT, &extent))
		    {
			printf ("  . mask extent: (%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPGetField (psp, PSPF_LAYERMASKDATAEXTENT, &extent))
		    {
			printf ("  . saved mask extent: "
				"(%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPGetField (psp, PSPF_LAYERMASKLINKED, &t_int))
		    {
			printf ("  . mask linked: %s\n",
				(t_int ? "Yes" : "No"));
		    }
		    if (PSPGetField (psp, PSPF_LAYERMASKDISABLED, &t_int))
		    {
			printf ("  . mask disabled: %s\n",
				(t_int ? "Yes" : "No"));
		    }
		    if (PSPGetField (psp, PSPF_LAYERMASKINVERT, &t_int))
		    {
			printf ("  . mask invert on blend: %s\n",
				(t_int ? "Yes" : "No"));
		    }
		    if (PSPGetField (psp, PSPF_LAYERBITMAPCOUNT, &t_u16))
			printf ("  . number of bitmaps: %d\n", t_u16);
		    if (PSPGetField (psp, PSPF_LAYERCHANNELCOUNT, &t_u16))
			printf ("  . number of channels: %d\n", t_u16);

		    if (PSPSetChannel (psp, PSP_CHANNELRED))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . red channel nr: %ld\n", t_32);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELGREEN))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . green channel nr: %ld\n", t_32);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELBLUE))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . blue channel nr: %ld\n", t_32);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELCOMPOSITE))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . grey/indexed channel nr: %ld\n", t_32);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELTRANSMASK))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . transparency channel nr: %ld\n", t_32);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELUSERMASK))
		    {
			PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
			printf ("  . user mask channel nr: %ld\n", t_32);
		    }
		    
		    if (PSPSetChannel (psp, PSP_CHANNELFIRST))
		    {
			do {
			    testread_channel (psp);
			} while (PSPNextChannel (psp));
		    }
		} while (PSPNextLayer (psp));
	    }
	    else
	    {
		printf ("Something wrong at layer selection!\n");
	    }
	    
	    printf ("Selection\n");
	    if (PSPGetField (psp, PSPF_SELECTIONEXTENT, &extent))
	    {
		printf ("  . extent: (%ld, %ld), (%ld, %ld)\n",
			extent.xmin, extent.ymin,
			extent.xmax, extent.ymax);
	    }
	    if (PSPGetField (psp, PSPF_SELECTIONDATAEXTENT, &extent))
	    {
		printf ("  . saved extent: (%ld, %ld), (%ld, %ld)\n",
			extent.xmin, extent.ymin,
			extent.xmax, extent.ymax);
	    }
	    
	    if (PSPGetField (psp, PSPF_SELECTIONBITMAPCOUNT, &t_u16))
		printf ("  . number of bitmaps: %d\n", t_u16);
	    if (PSPGetField (psp, PSPF_SELECTIONCHANNELCOUNT, &t_u16))
		printf ("  . number of channels: %d\n", t_u16);
	    
	    if (PSPSetChannel (psp, PSP_CHANNELSELECTION))
	    {
		PSPGetField (psp, PSPF_CHANNELNUMBER, &t_32);
		printf ("  . (first) channel nr: %ld\n", t_32);
	    }
	    
	    if (PSPSetChannel (psp, PSP_CHANNELSELECTIONFIRST))
	    {
		do {
		    testread_channel (psp);
		} while (PSPNextChannel (psp));
	    }
	    
	    if (PSPSetLayer (psp, PSP_LAYERFIRSTALPHA))
	    {
		do {
		    PSPGetField (psp, PSPF_LAYERNUMBER, &t_32);
		    printf ("Layer %ld\n", t_32);
		    if (PSPGetField (psp, PSPF_LAYERNAME, &string))
		    {
			printf ("  . name: %s\n", string);
			PSPFree (string);
		    }
		    if (PSPGetField (psp, PSPF_LAYEREXTENT, &extent))
		    {
			printf ("  . extent: (%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPGetField (psp, PSPF_LAYERDATAEXTENT, &extent))
		    {
			printf ("  . saved extent: (%ld, %ld), (%ld, %ld)\n",
				extent.xmin, extent.ymin,
				extent.xmax, extent.ymax);
		    }
		    if (PSPSetChannel (psp, PSP_CHANNELALPHAFIRST))
		    {
			do {
			    testread_channel (psp);
			} while (PSPNextChannel (psp));
		    }
		} while (PSPNextLayer (psp));
	    }

	    PSPClose (psp);
	}
    }
    return;
}

void
testwrite (int count, char * mode, char ** names)
{
    PSP * psp = NULL;
    int i;
    
    for (i = 0; i < count; i++)
    {
	printf ("==== Next file: %s\n", names[i]);
	
	if ((psp = PSPOpen (names[i], mode)) == NULL)
	{
	    printf ("Error opening file %s for writing!\n", names[i]);
	}
	else
	{
	    printf ("Fine! Opened file %s for writing\n", names[i]);
	    PSPClose (psp);
	}
    }
    return;
}

int
main (int argc, char ** argv) 
{
    if (argc < 3)
    {
	fprintf (stderr, "Usage: %s <psp debuglevel> <mode> <filename> ...\n",
		 argv [0]);
	return 1;
    }

    PSPDebug (atoi (argv[1]), 0);

    switch (argv[2][0])
    {
      case 'r':
	  testread (argc - 3, argv[2], &argv[3]);
	  break;
      case 'w':
	  testwrite (argc - 3, argv[2], &argv[3]);
	  break;
      default:
	  fprintf (stderr, "Usage: %s <psp debuglevel> <mode> <filename> ...\n",
		   argv [0]);
	  return 1;
    }
    

    return 0;
}
